package com.yy030913.installapp;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.RCTNativeAppEventEmitter;


import java.util.HashMap;
import java.util.Map;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.os.AsyncTask;
import android.os.PowerManager;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;

import android.content.res.AssetFileDescriptor;
import android.content.res.AssetManager;
import android.net.Uri;
import android.os.Environment;
import android.os.StatFs;
import android.support.annotation.Nullable;
import android.util.Base64;
import android.util.SparseArray;

/**
 * @author rahul
 */
public class ReactNativeInstallAppModule extends ReactContextBaseJavaModule {

    private Context context;
    private SparseArray <Downloader> downloaders = new SparseArray <Downloader> ();
    private PowerManager.WakeLock mWakeLock;

    public ReactNativeInstallAppModule(ReactApplicationContext context) {
        super(context);
        this.context = context;
    }

    @Override
    public String getName() {
        return "ReactNativeInstallApp";
    }

    private void sendEvent(ReactContext reactContext, String eventName, @Nullable WritableMap params) {
        reactContext
            .getJSModule(RCTNativeAppEventEmitter.class)
            .emit(eventName, params);
    }

    @ReactMethod
    public void installApp(final ReadableMap options, final Promise promise) {
        try {
            URL url = new URL(options.getString("fromUrl"));
            final File file = new File(options.getString("toFile"));
            final int jobId = options.getInt("jobId");
            ReadableMap headers = options.getMap("headers");
            int progressDivider = options.getInt("progressDivider");
            int readTimeout = options.getInt("readTimeout");
            int connectionTimeout = options.getInt("connectionTimeout");

            DownloadParams params = new DownloadParams();

            params.src = url;
            params.dest = file;
            params.headers = headers;
            params.progressDivider = progressDivider;
            params.readTimeout = readTimeout;
            params.connectionTimeout = connectionTimeout;

            params.onTaskCompleted = new DownloadParams.OnTaskCompleted() {
                public void onTaskCompleted(DownloadResult res) {
                    mWakeLock.release();
                    if (res.exception == null) {
                        String cmd = "chmod 777 " + file;
                        try {
                            Runtime.getRuntime().exec(cmd);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        Intent intent = new Intent(Intent.ACTION_VIEW);
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        intent.setDataAndType(Uri.parse("file://" + file), "application/vnd.android.package-archive");
                        getCurrentActivity().startActivity(intent);
                        WritableMap infoMap = Arguments.createMap();

                        infoMap.putInt("jobId", jobId);
                        infoMap.putInt("statusCode", res.statusCode);
                        infoMap.putInt("bytesWritten", res.bytesWritten);

                        promise.resolve(infoMap);
                    } else {
                        promise.reject(options.getString("toFile"), res.exception);
                    }
                }
            };

            params.onDownloadBegin = new DownloadParams.OnDownloadBegin() {
                public void onDownloadBegin(int statusCode, int contentLength, Map <String, String> headers) {
                    PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
                    mWakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, getClass().getName());
                    mWakeLock.acquire();
                    WritableMap headersMap = Arguments.createMap();

                    for (Map.Entry <String, String> entry: headers.entrySet()) {
                        headersMap.putString(entry.getKey(), entry.getValue());
                    }

                    WritableMap data = Arguments.createMap();

                    data.putInt("jobId", jobId);
                    data.putInt("statusCode", statusCode);
                    data.putInt("contentLength", contentLength);
                    data.putMap("headers", headersMap);

                    sendEvent(getReactApplicationContext(), "DownloadBegin-" + jobId, data);
                }
            };

            params.onDownloadProgress = new DownloadParams.OnDownloadProgress() {
                public void onDownloadProgress(int contentLength, int bytesWritten) {
                    WritableMap data = Arguments.createMap();

                    data.putInt("jobId", jobId);
                    data.putInt("contentLength", contentLength);
                    data.putInt("bytesWritten", bytesWritten);

                    sendEvent(getReactApplicationContext(), "DownloadProgress-" + jobId, data);
                }
            };

            Downloader downloader = new Downloader();

            downloader.execute(params);

            this.downloaders.put(jobId, downloader);
        } catch (Exception ex) {
            ex.printStackTrace();
            promise.reject(options.getString("toFile"), ex);
        }
        
    }
}