/**
 * Created by Rahul Jiresal on 01/22/16.
 */

'use strict';

var React = require('react-native');
import {
	Platform
} from 'react-native'
var NativeInstallApp = React.NativeModules.ReactNativeInstallApp;

var NativeAppEventEmitter = require('react-native').NativeAppEventEmitter; // iOS
var DeviceEventEmitter = require('react-native').DeviceEventEmitter; // Android

var jobId = 0;

var getJobId = () => {
	jobId += 1;
	return jobId;
};

type DownloadResult = {
	jobId: number; // The download job ID, required if one wishes to cancel the download. See `stopDownload`.
	statusCode: number; // The HTTP status code
	bytesWritten: number; // The number of bytes written to the file
};

module.exports = {
	installApp(options: DownloadAppOptions): {
		jobId: number,
		promise: Promise <DownloadResult>
	} {
		if (typeof options !== 'object') throw new Error('downloadFile: Invalid value for argument `options`');
		if (typeof options.fromUrl !== 'string') throw new Error('downloadFile: Invalid value for property `fromUrl`');
		if (typeof options.toFile !== 'string') throw new Error('downloadFile: Invalid value for property `toFile`');
		if (options.headers && typeof options.headers !== 'object') throw new Error('downloadFile: Invalid value for property `headers`');
		if (options.progressDivider && typeof options.progressDivider !== 'number') throw new Error('downloadFile: Invalid value for property `progressDivider`');
		if (options.readTimeout && typeof options.readTimeout !== 'number') throw new Error('downloadFile: Invalid value for property `readTimeout`');
		if (options.connectionTimeout && typeof options.connectionTimeout !== 'number') throw new Error('downloadFile: Invalid value for property `connectionTimeout`');

		var jobId = getJobId();
		var subscriptions = [];

		if (options.begin) {
			subscriptions.push(NativeAppEventEmitter.addListener('DownloadBegin-' + jobId, options.begin));
		}

		if (options.progress) {
			subscriptions.push(NativeAppEventEmitter.addListener('DownloadProgress-' + jobId, options.progress));
		}

		if (options.resumable) {
			subscriptions.push(NativeAppEventEmitter.addListener('DownloadResumable-' + jobId, options.resumable));
		}

		var bridgeOptions = {
			jobId: jobId,
			fromUrl: options.fromUrl,
			toFile: normalizeFilePath(options.toFile),
			headers: options.headers || {},
			progressDivider: options.progressDivider || 0,
			readTimeout: options.readTimeout || 15000,
			connectionTimeout: options.connectionTimeout || 5000
		};
		
		if (Platform.OS === 'android') {
			return {
				jobId,
				promise: NativeInstallApp.installApp(bridgeOptions).then(res => {
					subscriptions.forEach(sub => sub.remove());
					return res;
				})
			};
		}
	},

	installAppFromStore(options: DownloadAppOptions): {}{
		if (typeof options !== 'object') throw new Error('downloadFile: Invalid value for argument `options`');
		if (typeof options.fromUrl !== 'string') throw new Error('downloadFile: Invalid value for property `fromUrl`');
		if (typeof options.toFile !== 'string') throw new Error('downloadFile: Invalid value for property `toFile`');
		if (options.headers && typeof options.headers !== 'object') throw new Error('downloadFile: Invalid value for property `headers`');
		if (options.progressDivider && typeof options.progressDivider !== 'number') throw new Error('downloadFile: Invalid value for property `progressDivider`');
		if (options.readTimeout && typeof options.readTimeout !== 'number') throw new Error('downloadFile: Invalid value for property `readTimeout`');
		if (options.connectionTimeout && typeof options.connectionTimeout !== 'number') throw new Error('downloadFile: Invalid value for property `connectionTimeout`');

		var jobId = getJobId();

		var bridgeOptions = {
			jobId: jobId,
			fromUrl: options.fromUrl,
			toFile: normalizeFilePath(options.toFile),
			headers: options.headers || {},
			progressDivider: options.progressDivider || 0,
			readTimeout: options.readTimeout || 15000,
			connectionTimeout: options.connectionTimeout || 5000
		};
		
		if (Platform.OS === 'ios') {
			NativeInstallApp.installAPPFromStore(bridgeOptions)
		}
	}
};