//
//  ReactNativeInstallApp.m
//  ReactNativeInstallApp
//
//  Created by yy030913 on 11/23/17.
//  Copyright © 2017 yy030913. All rights reserved.
//

#import "Downloader.h"
#import "ReactNativeInstallApp.h"
#import <React/RCTBridge.h>

@implementation ReactNativeInstallApp

RCT_EXPORT_METHOD(installApp:(NSDictionary *)options resolver:(RCTPromiseResolveBlock)resolve rejecter:(RCTPromiseRejectBlock)reject)
{
    RNINAPPDownloadParams* params = [RNINAPPDownloadParams alloc];
    
    NSNumber* jobId = options[@"jobId"];
    params.fromUrl = options[@"fromUrl"];
    params.toFile = options[@"toFile"];
    NSDictionary* headers = options[@"headers"];
    params.headers = headers;
    NSNumber* discretionary = options[@"discretionary"];
    params.discretionary = [discretionary boolValue];
    NSNumber* progressDivider = options[@"progressDivider"];
    params.progressDivider = progressDivider;
    NSNumber* readTimeout = options[@"readTimeout"];
    params.readTimeout = readTimeout;
    
    __block BOOL callbackFired = NO;
    
    params.completeCallback = ^(NSNumber* statusCode, NSNumber* bytesWritten) {
        if (callbackFired) {
            return;
        }
        callbackFired = YES;
        
        NSMutableDictionary* result = [[NSMutableDictionary alloc] initWithDictionary: @{@"jobId": jobId}];
        if (statusCode) {
            [result setObject:statusCode forKey: @"statusCode"];
        }
        if (bytesWritten) {
            [result setObject:bytesWritten forKey: @"bytesWritten"];
        }
        return resolve(result);
    };
    
    params.errorCallback = ^(NSError* error) {
        if (callbackFired) {
            return;
        }
        callbackFired = YES;
        return [self reject:reject withError:error];
    };
    
    params.beginCallback = ^(NSNumber* statusCode, NSNumber* contentLength, NSDictionary* headers) {
        [self.bridge.eventDispatcher sendAppEventWithName:[NSString stringWithFormat:@"DownloadBegin-%@", jobId]
                                                     body:@{@"jobId": jobId, @"statusCode": statusCode, @"contentLength": contentLength, @"headers": headers ?: [NSNull null]}];
    };
    
    params.progressCallback = ^(NSNumber* contentLength, NSNumber* bytesWritten) {
        [self.bridge.eventDispatcher sendAppEventWithName:[NSString stringWithFormat:@"DownloadProgress-%@", jobId]
                                                     body:@{@"jobId": jobId, @"contentLength": contentLength, @"bytesWritten": bytesWritten}];
    };
    
    params.resumableCallback = ^() {
        [self.bridge.eventDispatcher sendAppEventWithName:[NSString stringWithFormat:@"DownloadResumable-%@", jobId] body:nil];
    };
    
    if (!self.downloaders) self.downloaders = [[NSMutableDictionary alloc] init];
    
    RNFSDownloader* downloader = [RNFSDownloader alloc];
    
    NSString *uuid = [downloader downloadFile:params];
    
    [self.downloaders setValue:downloader forKey:[jobId stringValue]];
    if (uuid) {
        if (!self.uuids) self.uuids = [[NSMutableDictionary alloc] init];
        [self.uuids setValue:uuid forKey:[jobId stringValue]];
    }
}

RCT_EXPORT_METHOD(installAppFromStore:(NSDictionary *)options)
{
    NSURL* url = [NSURL URLWithString:_params.fromUrl];
    UIApplication *application = [UIApplication sharedApplication];
    
    if ([application respondsToSelector:@selector(openURL:options:completionHandler:)]) {
        [application openURL:URL options:@{} completionHandler:nil];
    } else {
        [application openURL:URL];
    }
}
@end
